#!/usr/bin/env/python
# -*- coding:utf-8 -*-

import re


def get_item_head_info(items_str):
    """
    从试卷的题型行获取项目相关信息
    items_str:试题文本
    :return: 相关信息 和 按题型行切分后的大块
    """
    pattern0 = re.compile(r"\n\s*([一二三四五六七八九十]{,2})\s*[、.．､]\s*([^必考基础综合中等]{2,4}题)(.*?)\n")  # no 非
    all_type_info = re.findall(pattern0, items_str)
    all_type_info_idx = [[m.start(), m.end()] for m in re.finditer(pattern0, items_str)]  # 题型行位置
    if all_type_info and re.search("([\s。]+|[\u4e00-\u9fa5]\s*\.)1\s*[.．、､].+?", all_type_info[0][2]):
        if re.search("(^[^1]{,3}(<imgsrc((?!/>).)*?/>\s*)*?\n|^)\s*A[.．、､]", items_str[all_type_info_idx[0][1]:], flags=re.S):
            type1_info = re.search("([\s。]+|[\u4e00-\u9fa5]\s*\.)1\s*[.．、､].+?",
                                   items_str[all_type_info_idx[0][0]:all_type_info_idx[0][1]])
            all_type_info_idx[0][1] = type1_info.start() + len(type1_info.group(1))
    print("all_type_info_idx:",all_type_info_idx)
    if all_type_info_idx and all_type_info_idx[0][0] > 500:  # 第一大题题型漏了的情况
        insert_id = 0
        judge_right = 0
        for v in re.finditer(r"\n\s*([^必考基础综合中等：:()（）例训某课】]{,3})\s*[、.．､]\s*([^必考基础综合中等例训创某课】：:]{2,4}题)(.*?)\n",
                             items_str[:all_type_info_idx[0][0]]):  # 不严谨！！！！
            all_type_info.insert(insert_id, v.groups())
            all_type_info_idx.insert(insert_id, (v.start(), v.end()))
            insert_id += 1
            judge_right = 1
        if not judge_right:
            all_type_info.insert(insert_id, ["一", "选择题", ""])
            all_type_info_idx.insert(insert_id, (0, 0))

    # 统计每题得分
    each_item_score = []
    for a in all_type_info:
        score_info = re.search(r"每个?小?([题空])[\u4e00-\u9fa5]{,5}(\d[\d.]{,2}\d{,2})分", a[2])
        if score_info:
            if score_info.group(1) == '题' or '填空' in a[1]:
                each_item_score.append(float(score_info.group(2)))
            else:
                each_item_score.append(0.0)
        else:
            each_item_score.append(0.0)

    pattern1 = re.compile(r"[,，.第](\d+)题[^\d]{,5}(\d[\d.]{,2}\d{,2})分")
    pattern2 = re.compile(r"[,，.第](\d+)题?([,，、]\d+题?[,，、])*?[,，、~～](\d+)题?为选[做考]题")
    pattern3 = re.compile(r"[共计有包含括]+?(\d+)个?[小大]题.*?共(\d[\d.]{,2}\d{,2})分|[共计有]+?(\d+)个?[小大]题")
    pattern4 = re.compile(r"共[计有]*?(\d[\d.]{,2}\d{,2})分")
    each_item_score2 = {}  # 第xx题xx分
    select_type_id = []  # 选做题id   第xx题为选做题
    title_type_num = []  # 题型行中写明的题目个数，否则为0
    choice_class = {}
    all_type = []  # 所有题型
    total_score = []
    for k, a in enumerate(all_type_info):
        cur_type = a[1]
        if '本大题' in cur_type:
            if '选项' in a[2]:
                cur_type = '选择题'
            elif re.search("每空|填空|空格", a[2]):
                cur_type = '填空题'
            else:
                cur_type = '解答题'
        all_type.append(cur_type)

        if a[2] and re.search(pattern1, a[2]):
            each_item_score2.update({i[0]: float(i[1]) for i in re.findall(pattern1, a[2])})
        if a[2] and re.search(pattern2, a[2]):
            st1 = re.search(pattern2, a[2]).group(1)
            ed1 = re.search(pattern2, a[2]).group(3)
            if int(ed1) - int(st1) > 0:  # 现只考虑2个以上选做题的情况
                select_type_id.extend(list(range(int(st1), int(ed1) + 1)))
        if a[2] and re.search(pattern3, a[2]):
            info3 = re.search(pattern3, a[2])
            type_num = (str(info3.group(1)) + str(info3.group(3))).replace('None', '')
            title_type_num.append([cur_type, int(type_num)])
            if info3.group(2) and not each_item_score[k] and \
                    all_type[k].replace("题", "") in ["单选", "不定选择", "多选", "选择"]:
                each_item_score[k] = float(info3.group(2)) / int(type_num)
        else:
            title_type_num.append([cur_type, 0])
        if a[2] and re.search(pattern4, a[2]):
            info4 = re.search(pattern4, a[2])
            total_score.append(float(info4.group(1)))
        else:
            total_score.append(0.0)

        # 从题型行中判断单选和多选
        if cur_type == "选择题":
            multi_choice_info = re.findall("[\s,，；;(（]+第?(\d+)[至到\-~]+(\d+)题[是为]([多单])项?选择?题?", a[2])
            if multi_choice_info:
                for mu in multi_choice_info:
                    choice_class[mu[2]] = list(range(int(mu[0]), int(mu[1]) + 1))


    print("each_item_score:", each_item_score)
    print("each_item_score2:", each_item_score2)
    print("select_type_id:", select_type_id)
    print("title_type_num:", title_type_num)
    print("all_type_info_idx:", all_type_info_idx)
    print("multi_choice_info:", choice_class)

    # 按题型行切分大块
    con_block = []
    if all_type_info_idx:
        for k, m in enumerate(all_type_info_idx):
            if k == 0:
                con_block.append(items_str[0:m[0]])
            else:
                con_block.append(items_str[all_type_info_idx[k - 1][1]:m[0]])
            if k == len(all_type_info_idx) - 1:
                con_block.append(items_str[m[1]:])

    else:
        con_block.append(items_str)
    # 切分后的初步清洗
    if all_type_info and all_type_info[0][0] == "一":
        del con_block[0]
    else:
        while (con_block and (re.search(r"[\u4e00-\u9fa5]", con_block[0]) is None or
                          re.search(r"\n\s*([1-9]|[1-2][0-9])\s*[.．、､]", con_block[0]) is None)) \
                or (len(con_block) > 1 and re.search(r"\n\s*1\s*[.．、､]", con_block[1])):  # 第2个条件在特殊情况下不太妥当
            del con_block[0]
    if len(con_block)==len(all_type)+1:
        total_score.insert(0,0.0)
    else:
        total_score = [0.0]*len(con_block)

    title_info_dict = {"all_type":all_type, "each_item_score":each_item_score, "each_item_score2":each_item_score2,
                       "select_type_id":select_type_id, "title_type_num":title_type_num, "total_score":total_score}

    return con_block, title_info_dict, choice_class


